#!/usr/bin/env python

import wnix

def object_gettext(object):

    import inspect

    methods = [
        inspect.getsource,
        inspect.getdoc,
        lambda object: object.__qualname__,
        lambda object: object.__name__,
    ]

    for method in methods:
        try:
            text = method(object)
            if text is None: raise ValueError
            return text
        except:
            pass
    else:
        return repr(object)

def module_getsource(module):
    texts = {}
    for name in dir(module):
        object = getattr(module, name)
        text = object_gettext(object)
        if text is None: continue
        texts[name] = text
    return texts


def module_getsource_for_classes(module):
    texts = {}
    for name in dir(module):
        object = getattr(module, name)
        if not isinstance(object, type):
            continue
        texts[name] = object_gettext(object)
    return texts

def test_grep_understands_python_module_tempfile():

    import tempfile
    d = module_getsource(tempfile)

    g = wnix.grep('Make a directory', d, n=2)
    i = g.intersect({'tempdir', 'TemporaryDirectory'})
    assert len(i) >= 1


def test_grep_understands_python_module_asyncio():

    import asyncio
    d = module_getsource(asyncio)

    g = wnix.grep('Run a thing', d, n=2)
    assert len(g.intersect({'run'})) >= 1

    g = wnix.grep('Run a coroutine from another thread', d, n=2)
    assert len(g.intersect({'run_coroutine_threadsafe'})) >= 1

    g = wnix.grep('Concurrency primitives', d, n=2)
    assert len(g.intersect({'locks'})) >= 1


def test_grep_understands_python_module_threading():

    import threading
    d = module_getsource_for_classes(threading)

    g = wnix.grep("A re-entrant lock", d, n=2)
    assert len(g.intersect({'RLock', '_RLock'})) >= 1

    g = wnix.grep("A condition variable", d, n=2)
    assert len(g.intersect({'Condition'})) >= 1

    g = wnix.grep("Semaphore released too many times", d, n=2)
    assert len(g.intersect({'BoundedSemaphore'})) >= 1

    g = wnix.grep("Has a .set(), .clear(), and .wait() method", d, n=2)
    assert len(g.intersect({'Event'})) >= 1




#!/usr/bin/env python3

import embd
import assure
import pathlib
from drivers import pdf
from drivers import image

IMAGES = [
    'root/usr/share/cats.jpg',
    'root/usr/share/bluecow.jpg',
    'root/usr/share/redcow.jpg',
    'root/usr/share/soccer.jpg',
    'root/usr/share/cooked.jpg',
    'root/usr/share/chicken.jpg',
    'root/usr/share/chick.jpg',
]

PDFS = [
    'root/usr/share/chicken.pdf',
    'root/usr/share/cow.pdf',
]

class File:
    def __init__(self, file):
        self.file  = pathlib.Path(file)
        self.bytes = assure.bytes(open(file, 'rb'))

    def __repr__(self):
        cls = self.__class__.__name__
        name = self.file.as_posix()
        return f"{cls}({name})"

    def embed(self):
        return embd.think(self.text())

    def name(self):
        return self.file.name

    def stem(self):
        return self.file.stem

class Image(File):
    def text(self):
        text1 = image.to_text(self.bytes)
        text2 = image.to_text(self.file)
        assert text1 == text2
        return text1

class ImageAndText(Image):
    def text(self, query):
        text1 = image.and_text_to_text(self.bytes, query)
        text2 = image.and_text_to_text(self.file, query)
        assert text1 == text2
        return text1

class Pdf(File):
    def text(self):
        text1 = pdf.to_text(self.bytes)
        text2 = pdf.to_text(self.file)
        assert text1 == text2
        return text1


def get_pdfs():
    return [Pdf(file) for file in PDFS]

def get_images():
    return [Image(file) for file in IMAGES]

def get_files():
    return get_pdfs() + get_images()

def get_texts():
    files = get_files()
    return [file.text() for file in files]

def get_embeds():
    files = get_files()
    return [file.embed() for file in files]

def test_image_to_text_correctness():
    images = {p.stem(): p for p in get_images()}
    assert 'cat'    in images['cats'].text()
    assert 'blue'   in images['bluecow'].text()
    assert 'cow'    in images['bluecow'].text()
    assert 'red'    in images['redcow'].text()
    assert 'cow'    in images['redcow'].text()
    assert 'soccer' in images['soccer'].text()

def test_pdf_to_text_correctness():
    pdfs = {p.stem(): p for p in get_pdfs()}
    assert 'chicken' in pdfs['chicken'].text()
    assert 'CoW'     in pdfs['cow'].text()



#!/usr/bin/python3

from wnix import grep

def foo(n):
    while True:
        n += 1
    return n

def bar(n):
    s = 0
    for k in range(n):
        s += k
    return s

def baz(n):
    while n > 0:
        n -= 1
    return n

def doo(n):
    n = abs(n)+1
    for j in range(n):
        pass
    return j

def boo(n):
    x = n
    for k in range(n**2):
        x += k**2
    return x


def test_simple_halting():
    import inspect
    functions  = (foo, bar, baz, doo, boo)
    sources    = [inspect.getsource(f) for f in functions]
    names      = [f.__name__ for f in functions]

    answers    = ['for loop', 'while loop']
    g = grep(sources, answers, n=1)
    assert g[1].tolist() == [
        'while loop', 'for loop', 'while loop', 'for loop', 'for loop'
    ]




from wnix import what

def test_what():
    q = ['red cow', 'blue cow', 'green chicken', 'purple chicken']
    k = ['dog', 'cat', 'bird', 'snake', 'cow', 'chicken', 'horse', 'pig']
    w = what(q, k)
    assert w.ravel().tolist() == ['cow', 'cow', 'chicken', 'chicken']



#!/usr/bin/env python3

__all__ = ['main']

import os
import re
import sys
import shlex
import subprocess
import argparse

def run(cmd, input):
    p = subprocess.Popen(shlex.quote(cmd), shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    stdout, stderr = p.communicate(input.encode())
    return stdout.decode()

def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    parser = argparse.ArgumentParser('sort2')
    parser.add_argument('key')
    parser.add_argument('-c', '--cmd', type=str, default=None)
    parser.add_argument('-d', '--debug', action='store_true')
    args = parser.parse_args(argv)

    from embd import List, Dict

    lines = sys.stdin.read().splitlines()

    if args.cmd:
        qs = [run(args.cmd, line) for line in lines]
    else:
        qs = lines

    qs = {l:q for l,q in zip(lines, qs)}

    q = Dict(qs)
    k = List([args.key])
    s = q @ k
    if args.debug:
        print(f'=== q ===')
        import json
        print(json.dumps(q.o, indent=2))
        print(q)
        print(f'=== k ===')
        print(k)
        print(f'=== s ===')
        print(s)
        sys.exit(0)

    o = s[args.key].sort_values(ascending=False).index.tolist()
    for l in o:
        print(l)

if __name__ == '__main__':
    main(sys.argv[1:])
